@@ -53,21 +53,23 @@ def parse_single_player(obs_raw_entry):
     return obs_raw_entry
 
 
-def try_get_video(env):
+def try_get_video(env, keep_running=False):
     if not env.football_video_path:
         internal_env = m_envs[env.configuration.id]
         while not hasattr(internal_env, '_env'):
             internal_env = internal_env.env
-        if env.done and internal_env._env._step == -1:
+        if not keep_running and internal_env._env._step == -1:
             # Generate no-op step, so that video is available.
             internal_env.step([0] * (env.configuration.team_1 + env.configuration.team_2))
         trace = internal_env._env._trace
-        trace._dump_config['episode_done']._min_frequency = 0
-        dumps = trace.process_pending_dumps(True)
-        env.football_video_path = retrieve_video_link(dumps)
+        if trace:
+          trace._dump_config['episode_done']._min_frequency = 0
+          dumps = trace.process_pending_dumps(True)
+          env.football_video_path = retrieve_video_link(dumps)
         if not env.football_video_path:
             return
-        trace.write_dump('episode_done')
+        if keep_running:
+            trace.write_dump('episode_done')
     if 'LiveVideoPath' in env.info and env.info['LiveVideoPath'] is not None:
         target_path = Path(env.info['LiveVideoPath'])
         target_path.parent.mkdir(parents=True, exist_ok=True)
@@ -84,15 +86,8 @@ def update_observations_and_rewards(configuration, state, obs, rew=None):
 
     assert len(obs) == configuration.team_1 + configuration.team_2
     if rew is not None:
-        if type(rew) == np.float32:
-            rew = [rew]
-        assert len(rew) == configuration.team_1 + configuration.team_2
-        if configuration.team_1 > 0:
-            state[0].reward = float(rew[0])
-        else:
-            state[0].reward = float(-rew[0])
-        state[1].reward = -state[1].reward
-
+        state[0].reward = rew
+        state[1].reward = -rew
     state[0].observation.players_raw = [
         parse_single_player(obs[x]) for x in range(configuration.team_1)
     ]
@@ -227,7 +222,7 @@ def interpreter(state, env):
             (env.configuration.team_2, len(state[1].action)))
     if env.configuration.team_2:
         actions_to_env = actions_to_env + state[1].action
-    
+
     if maybe_terminate(env, state):
         return state
     obs, rew, done, info = m_envs[env.configuration.id].step(actions_to_env)
@@ -237,7 +232,7 @@ def interpreter(state, env):
     update_observations_and_rewards(configuration=env.configuration,
                                     state=state,
                                     obs=obs,
-                                    rew=rew)
+                                    rew=obs[0]['score'][0]-obs[0]['score'][1])
 
     ## TODO: pass other information from 'info' to the state/agent.
     if done:
@@ -255,7 +250,7 @@ with open(jsonpath) as f:
 
 
 def html_renderer(env):
-    try_get_video(env)
+    try_get_video(env, keep_running=True)
     if not env.football_video_path:
         raise Exception(
             "No video found. Was environment created with save_video enabled?"
