@@ -9,8 +9,8 @@ def line(x1: int, y1: int, x2: int, y2: int) -> str:
 
 # text at cell on map
 def text(x: int, y: int, message: str, fontsize: int = 16) -> str:
-    return f"dt {x} {y} {message} {fontsize}"
+    return f"dt {x} {y} {fontsize} '{message}'"
 
 # text besides map
 def sidetext(message: str) -> str:
-    return f"dst {message}"
+    return f"dst '{message}'"
