@@ -56,11 +56,12 @@ def env():
 
 
 class LuxAI2022(ParallelEnv):
-    metadata = {"render.modes": ["human", "html"], "name": "luxai2022_v0"}
+    metadata = {"render.modes": ["human", "html", "rgb_array"], "name": "luxai2022_v0"}
 
-    def __init__(self, **kwargs):
-        # TODO - allow user to override env configs
+    def __init__(self, collect_stats: bool = False, **kwargs):
+        self.collect_stats = collect_stats # note: added here instead of in configs since it would break existing bots
         default_config = EnvConfig(**kwargs)
+
         self.env_cfg = default_config
         self.possible_agents = ["player_" + str(r) for r in range(2)]
         self.agent_name_mapping = dict(zip(self.possible_agents, list(range(len(self.possible_agents)))))
@@ -79,7 +80,7 @@ class LuxAI2022(ParallelEnv):
         return get_obs_space(config=self.env_cfg, agent_names=self.possible_agents, agent=agent)
 
     # @functools.lru_cache(maxsize=None)
-    def action_space(self, agent):
+    def action_space(self, agent: str):
         if self.env_cfg.BIDDING_SYSTEM:
             if self.env_steps == 0:
                 # bid first, then place factories
@@ -92,21 +93,33 @@ class LuxAI2022(ParallelEnv):
                 return get_act_space_init(config=self.env_cfg, agent=agent)
             return get_act_space(self.state.units, self.state.factories, config=self.env_cfg, agent=agent)
 
-    def render(self, mode="human"):
+    def _init_render(self):
+        if self.py_visualizer is None:
+            self.py_visualizer = Visualizer(self.state)
+            return True
+        return False
+    def render(self, mode="human", **kwargs):
         """
         Renders the environment. In human mode, it can print to terminal, open
         up a graphical window, or open up some other display that a human can see and understand.
         """
-        # if len(self.agents) == 2:
-        #     string = ("Current state: Agent1: {} , Agent2: {}".format(MOVES[self.state[self.agents[0]]], MOVES[self.state[self.agents[1]]]))
-        # else:
-        #     string = "Game over"
 
         if mode == "human":
-            if self.py_visualizer is None:
-                self.py_visualizer = Visualizer(self.state)
+            if self._init_render():
+                self.py_visualizer.init_window()
+            
             self.py_visualizer.update_scene(self.state)
             self.py_visualizer.render()
+        elif mode == "rgb_array":
+            self._init_render()
+            self.py_visualizer.update_scene(self.state)
+            VIDEO_W = 400
+            VIDEO_H = 400
+            if "width" in kwargs:
+                VIDEO_W = kwargs["width"]
+            if "height" in kwargs:
+                VIDEO_H = kwargs["height"]
+            return self.py_visualizer._create_image_array(self.py_visualizer.surf, (VIDEO_W, VIDEO_H))
 
     def close(self):
         """
@@ -114,6 +127,12 @@ class LuxAI2022(ParallelEnv):
         or any other environment data which should not be kept around after the
         user is no longer using the environment.
         """
+        try:
+            import pygame
+            pygame.display.quit()
+            pygame.quit()
+        except:
+            print("No pygame installed, ignoring import")
         pass
 
     def get_state(self):
@@ -149,6 +168,12 @@ class LuxAI2022(ParallelEnv):
         for agent in self.possible_agents:
             self.state.units[agent] = OrderedDict()
             self.state.factories[agent] = OrderedDict()
+            if self.collect_stats:
+                self.state.stats[agent] = dict(
+                    lichen_grown=0, lichen_lost=0, 
+                    units_lost=dict(LIGHT=0, HEAVY=0), 
+                    factories_lost=0, units_built=dict(LIGHT=0, HEAVY=0)
+                )
         obs = self.state.get_obs()
         observations = {agent: obs for agent in self.agents}
         return observations
@@ -168,6 +193,10 @@ class LuxAI2022(ParallelEnv):
             if k not in self.agents:
                 raise ValueError(f"Invalid player {k}")
             if "faction" in a and "bid" in a:
+                if a["faction"] not in [e.name for e in FactionTypes]:
+                    self._log(f"{k} initialized with invalid faction name {a['faction']}")
+                    failed_agents[k] = True
+                    continue
                 self.state.teams[k] = Team(
                     team_id=self.agent_name_mapping[k], agent=k, faction=FactionTypes[a["faction"]]
                 )
@@ -176,8 +205,9 @@ class LuxAI2022(ParallelEnv):
                 self.state.teams[k].factories_to_place = self.state.board.factories_per_team
                 # verify bid is valid
                 valid_action = True
-                bid = a["bid"]
-                if bid < 0 or bid > self.state.teams[k].init_water:
+                bid = abs(a["bid"])
+                self.state.teams[k].bid = a["bid"]
+                if bid > self.state.teams[k].init_water:
                     valid_action = False
                 if not valid_action:
                     failed_agents[k] = True
@@ -186,23 +216,46 @@ class LuxAI2022(ParallelEnv):
                     highest_bid = bid
                     highest_bid_agent = k
                 elif bid == highest_bid:
-                    # no one gets the extra factory
-                    highest_bid_agent = "tie"
+                    # if bids are the same, player 0 defaults to the winner and pays the bid.
+                    highest_bid_agent = "player_0"
             else:
                 # team k loses
                 failed_agents[k] = True
-        if highest_bid_agent == "tie":
-            pass
-        elif highest_bid_agent is None:
+        if highest_bid_agent is None:
             # no valid bids made, all agents failed.
             pass
         else:
+            lowest_bid_agent = "player_1"
+            if highest_bid_agent == "player_1":
+                lowest_bid_agent = "player_0"
             self.state.teams[highest_bid_agent].init_water -= highest_bid
             self.state.teams[highest_bid_agent].init_metal -= highest_bid
-            self.state.teams[highest_bid_agent].factories_to_place += 1
+
+            # highest bid agent either won because of a tie, which then player_0 default wins, or because they had a stronger bid
+            # who ever wins will always get their choice
+            if self.state.teams[highest_bid_agent].bid < 0:
+                self.state.teams[highest_bid_agent].place_first = False
+                self.state.teams[lowest_bid_agent].place_first = True
+            else:
+                self.state.teams[highest_bid_agent].place_first = True
+                self.state.teams[lowest_bid_agent].place_first = False
         return failed_agents
     def _handle_factory_placement_step(self, actions):
-        # factory placement rounds
+        # factory placement rounds, which are sequential
+
+        player_to_place_factory: str
+        if self.state.teams["player_0"].place_first:
+            if self.state.env_steps % 2 == 1:
+                player_to_place_factory = "player_0"
+            else:
+                player_to_place_factory = "player_1"
+        else:
+            if self.state.env_steps % 2 == 1:
+                player_to_place_factory = "player_1"
+            else:
+                player_to_place_factory = "player_0"
+
+
         failed_agents = {agent: False for agent in self.agents}
         for k, a in actions.items():
             if a is None:
@@ -211,6 +264,9 @@ class LuxAI2022(ParallelEnv):
             if k not in self.agents:
                 raise ValueError(f"Invalid player {k}")
             if "spawn" in a and "metal" in a and "water" in a:
+                if k != player_to_place_factory:
+                    self._log(f"{k} tried to perform an action in the early phase when it is not its turn right now.")
+                    continue
                 if self.state.teams[k].factories_to_place <= 0:
                     self._log(f"{k} cannot place additional factories. Cancelled placement of factory")
                     continue
@@ -218,12 +274,11 @@ class LuxAI2022(ParallelEnv):
                     self._log(f"{k} tried to place negative water/metal in factory. Cancelled placement of factory")
                     continue
                 if a["water"] > self.state.teams[k].init_water:
-                    self._log(f"{k} does not have enough water. Cancelled placement of factory")
-                    continue
+                    a["water"] = self.state.teams[k].init_water
+                    self._log(f" Warning - {k} does not have enough water. Using {a['water']}")
                 if a["metal"] > self.state.teams[k].init_metal:
-                    self._log(f"{k} does not have enough metal. Cancelled placement of factory")
-                    continue
-
+                    a["metal"] = self.state.teams[k].init_water
+                    self._log(f" Warning - {k} does not have enough metal. Using {a['metal']}")
                 factory = self.add_factory(self.state.teams[k], a["spawn"])
                 if factory is None: continue
                 factory.cargo.water = a["water"]
@@ -275,35 +330,39 @@ class LuxAI2022(ParallelEnv):
             units_there = self.state.board.get_units_at(transfer_pos)
 
             # if there is a factory, we prefer transferring to that entity
-            factory_id = f"factory_{self.state.board.factory_occupancy_map[transfer_pos.y, transfer_pos.x]}"
+            factory_id = f"factory_{self.state.board.factory_occupancy_map[transfer_pos.x, transfer_pos.y]}"
             if factory_id in self.state.factories[unit.team.agent]:
                 factory = self.state.factories[unit.team.agent][factory_id]
-                actually_transferred = factory.add_resource(transfer_action.resource, transfer_action.transfer_amount)
+                actually_transferred = factory.add_resource(transfer_action.resource, transfer_amount)
             elif units_there is not None:
                 assert len(units_there) == 1, "Fatal error here, this is a bug"
                 target_unit = units_there[0]
                 # add resources to target. This will waste (transfer_amount - actually_transferred) resources
                 actually_transferred = target_unit.add_resource(transfer_action.resource, transfer_amount)
+            unit.repeat_action(transfer_action)
+
     def _handle_pickup_actions(self, actions_by_type: ActionsByType):
         for unit, pickup_action in actions_by_type["pickup"]:
             pickup_action: PickupAction
-            factory = self.state.board.get_factory_at(unit.pos)
+            factory = self.state.board.get_factory_at(self.state, unit.pos)
             pickup_amount = factory.sub_resource(pickup_action.resource, pickup_action.pickup_amount)
             # may waste resources if tried to pickup more than one can hold.
             actually_pickedup = unit.add_resource(pickup_action.resource, pickup_amount)
+            unit.repeat_action(pickup_action)
 
     def _handle_dig_actions(self, actions_by_type: ActionsByType, weather_cfg):
         for unit, dig_action in actions_by_type["dig"]:
             dig_action: DigAction
-            if self.state.board.rubble[unit.pos.y, unit.pos.x] > 0:
-                self.state.board.rubble[unit.pos.y, unit.pos.x] = max(self.state.board.rubble[unit.pos.y, unit.pos.x] - unit.unit_cfg.DIG_RUBBLE_REMOVED, 0)
-            elif self.state.board.lichen[unit.pos.y, unit.pos.x] > 0:
-                self.state.board.lichen[unit.pos.y, unit.pos.x] = max(self.state.board.lichen[unit.pos.y, unit.pos.x] - unit.unit_cfg.DIG_LICHEN_REMOVED, 0)
-            elif self.state.board.ice[unit.pos.y, unit.pos.x] > 0:
+            if self.state.board.rubble[unit.pos.x, unit.pos.y] > 0:
+                self.state.board.rubble[unit.pos.x, unit.pos.y] = max(self.state.board.rubble[unit.pos.x, unit.pos.y] - unit.unit_cfg.DIG_RUBBLE_REMOVED, 0)
+            elif self.state.board.lichen[unit.pos.x, unit.pos.y] > 0:
+                self.state.board.lichen[unit.pos.x, unit.pos.y] = max(self.state.board.lichen[unit.pos.x, unit.pos.y] - unit.unit_cfg.DIG_LICHEN_REMOVED, 0)
+            elif self.state.board.ice[unit.pos.x, unit.pos.y] > 0:
                 unit.add_resource(0, unit.unit_cfg.DIG_RESOURCE_GAIN)
-            elif self.state.board.ore[unit.pos.y, unit.pos.x] > 0:
+            elif self.state.board.ore[unit.pos.x, unit.pos.y] > 0:
                 unit.add_resource(1, unit.unit_cfg.DIG_RESOURCE_GAIN)
             unit.power -= math.ceil(self.state.env_cfg.ROBOTS[unit.unit_type.name].DIG_COST * weather_cfg["power_loss_factor"])
+            unit.repeat_action(dig_action)
     def _handle_self_destruct_actions(self, actions_by_type: ActionsByType):
         for unit, self_destruct_action in actions_by_type["self_destruct"]:
             unit: Unit
@@ -311,22 +370,44 @@ class LuxAI2022(ParallelEnv):
             pos_hash = self.state.board.pos_hash(unit.pos)
             del self.state.board.units_map[pos_hash]
             self.destroy_unit(unit)
+            if self.collect_stats:
+                self.state.stats[unit.team.agent]["units_lost"][unit.unit_type.name] += 1
+
     def _handle_factory_build_actions(self, actions_by_type: ActionsByType, weather_cfg):
         for factory, factory_build_action in actions_by_type["factory_build"]:
             factory: Factory
             factory_build_action: FactoryBuildAction
             team = self.state.teams[factory.team.agent]
+
+            # note that this happens after unit resource pickup. Thus we have to reverify these actions
+            # if self.state.env_cfg.validate_action_space:
+
+            if factory_build_action.unit_type == UnitType.HEAVY:
+                if factory.cargo.metal < self.env_cfg.ROBOTS["HEAVY"].METAL_COST:
+                    self._log(f"{factory} doesn't have enough metal to build a heavy despite having enough metal at the start of the turn. This is likely because a unit picked up some of the metal.")
+                    return
+                if factory.power < factory_build_action.power_cost:
+                    self._log(f"{factory} doesn't have enough power to build a heavy despite having enough power at the start of the turn. This is likely because a unit picked up some of the power.")
+                    return
+                factory.sub_resource(3, self.env_cfg.ROBOTS["HEAVY"].METAL_COST)
+                factory.sub_resource(4, factory_build_action.power_cost)
+            else:
+                if factory.cargo.metal < self.env_cfg.ROBOTS["LIGHT"].METAL_COST:
+                    self._log(f"{factory} doesn't have enough metal to build a light despite having enough metal at the start of the turn. This is likely because a unit picked up some of the metal.")
+                    return
+                if factory.power < factory_build_action.power_cost:
+                    self._log(f"{factory} doesn't have enough power to build a light despite having enough power at the start of the turn. This is likely because a unit picked up some of the power.")
+                    return
+                factory.sub_resource(3, self.env_cfg.ROBOTS["LIGHT"].METAL_COST)
+                factory.sub_resource(4, factory_build_action.power_cost)
+            
             self.add_unit(
                 team=team,
                 unit_type=factory_build_action.unit_type,
                 pos=factory.pos.pos,
             )
-            if factory_build_action.unit_type == UnitType.HEAVY:
-                factory.sub_resource(3, self.env_cfg.ROBOTS["HEAVY"].METAL_COST)
-                factory.sub_resource(4, math.ceil(self.env_cfg.ROBOTS["HEAVY"].POWER_COST * weather_cfg["power_loss_factor"]))
-            else:
-                factory.sub_resource(3, self.env_cfg.ROBOTS["LIGHT"].METAL_COST)
-                factory.sub_resource(4, math.ceil(self.env_cfg.ROBOTS["LIGHT"].POWER_COST * weather_cfg["power_loss_factor"]))
+            if self.collect_stats:
+                self.state.stats[factory.team.agent]["units_built"][factory_build_action.unit_type.name] += 1
     def _handle_movement_actions(self, actions_by_type: ActionsByType, weather_cfg):
         new_units_map: Dict[str, List[Unit]] = defaultdict(list)
         heavy_entered_pos: Dict[str, List[Unit]] = defaultdict(list)
@@ -335,29 +416,28 @@ class LuxAI2022(ParallelEnv):
         for unit, move_action in actions_by_type["move"]:
             move_action: MoveAction
             # skip move center
-            if move_action.move_dir == 0:
-                continue
-            old_pos_hash = self.state.board.pos_hash(unit.pos)
-            target_pos = unit.pos + move_action.dist * move_deltas[move_action.move_dir]
-            rubble = self.state.board.rubble[target_pos.y, target_pos.x]
-            power_required = unit.unit_cfg.MOVE_COST + unit.unit_cfg.RUBBLE_MOVEMENT_COST * rubble
-            power_required = math.ceil(power_required * weather_cfg["power_loss_factor"])
-            unit.pos = target_pos
-            new_pos_hash = self.state.board.pos_hash(unit.pos)
+            if move_action.move_dir != 0:
+                old_pos_hash = self.state.board.pos_hash(unit.pos)
+                target_pos = unit.pos + move_action.dist * move_deltas[move_action.move_dir]
+                power_required = move_action.power_cost
+                unit.pos = target_pos
+                new_pos_hash = self.state.board.pos_hash(unit.pos)
 
-            # remove unit from map temporarily
-            if len(self.state.board.units_map[old_pos_hash]) == 1:
-                del self.state.board.units_map[old_pos_hash]
-            else:
-                self.state.board.units_map[old_pos_hash].remove(unit)
+                # remove unit from map temporarily
+                if len(self.state.board.units_map[old_pos_hash]) == 1:
+                    del self.state.board.units_map[old_pos_hash]
+                else:
+                    self.state.board.units_map[old_pos_hash].remove(unit)
 
-            new_units_map[new_pos_hash].append(unit)
-            unit.power -= power_required
+                new_units_map[new_pos_hash].append(unit)
+                unit.power -= power_required
 
-            if unit.unit_type == UnitType.HEAVY:
-                heavy_entered_pos[new_pos_hash].append(unit)
-            else:
-                light_entered_pos[new_pos_hash].append(unit)
+                if unit.unit_type == UnitType.HEAVY:
+                    heavy_entered_pos[new_pos_hash].append(unit)
+                else:
+                    light_entered_pos[new_pos_hash].append(unit)
+
+            unit.repeat_action(move_action)
 
         for pos_hash, units in self.state.board.units_map.items():
             # add in all the stationary units
@@ -380,7 +460,7 @@ class LuxAI2022(ParallelEnv):
                 for u in units:
                     if u.unit_id != surviving_unit.unit_id:
                         destroyed_units.add(u)
-                self._log(f"{len(destroyed_units)} Units collided at {pos_hash} with {surviving_unit} surviving")
+                self._log(f"{len(destroyed_units)} Units: ({', '.join([u.unit_id for u in destroyed_units])}) collided at {pos_hash} with {surviving_unit} surviving")
                 new_units_map_after_collision[pos_hash].append(surviving_unit)
             else:
                 # check for stationary heavy unit there
@@ -418,21 +498,24 @@ class LuxAI2022(ParallelEnv):
 
         for u in destroyed_units:
             self.destroy_unit(u)
+            if self.collect_stats:
+                self.state.stats[u.team.agent]["units_lost"][u.unit_type.name] += 1
+
     def _handle_recharge_actions(self, actions_by_type: ActionsByType):
         # for recharging actions, check if unit has enough power. If not, add action back to the queue
         for unit, recharge_action in actions_by_type["recharge"]:
             recharge_action: RechargeAction
             if unit.power < recharge_action.power:
-                unit.action_queue.insert(0, recharge_action)
-                # by default actions with repeat=True will be placed to the back of queue
-                # remove from back of queue if we re-inserted into the front
-                if recharge_action.repeat:
-                    unit.action_queue = unit.action_queue[:-1]
+                pass
+            else:
+                # if unit got enough power, handle the action and consider it for repeating
+                unit.repeat_action(recharge_action)
     def _handle_factory_water_actions(self, actions_by_type: ActionsByType):
         for factory, factory_water_action in actions_by_type["factory_water"]:
+            factory_water_action: FactoryWaterAction
             water_cost = factory.water_cost(self.env_cfg)
             factory.cargo.water -= water_cost # earlier validation ensures this is always possible.
-            indexable_positions = ([v[1] for v in factory.grow_lichen_positions], [v[0] for v in factory.grow_lichen_positions])
+            indexable_positions = ([v[0] for v in factory.grow_lichen_positions], [v[1] for v in factory.grow_lichen_positions])
             self.state.board.lichen[indexable_positions] += 2
             self.state.board.lichen_strains[indexable_positions] = factory.num_id
     def step(self, actions):
@@ -455,7 +538,7 @@ class LuxAI2022(ParallelEnv):
         early_game = False
         if self.env_steps == 0:
             early_game = True
-        if self.env_cfg.BIDDING_SYSTEM and self.env_steps <= self.state.board.factories_per_team + 1:
+        if self.env_cfg.BIDDING_SYSTEM and self.state.real_env_steps < 0:
             early_game = True
 
         if early_game:
@@ -467,13 +550,14 @@ class LuxAI2022(ParallelEnv):
             weather_cfg = weather.apply_weather(self.state, self.agents, current_weather)
 
             # 1. Check for malformed actions
+            
             if self.env_cfg.validate_action_space:
-                # This part is not absolutely necessary
+                # This part is not absolutely necessary if you know for sure your actions are well formatted
                 for agent, unit_actions in actions.items():
                     valid_acts, err_reason = self.action_space(agent).contains(unit_actions)
                     if not valid_acts:
                         failed_agents[agent] = True
-                        raise ValueError(f"{self.state.teams[agent]} Inappropriate action given. {err_reason}")
+                        self._log(f"{self.state.teams[agent]} Inappropriate action given. {err_reason}")
 
             # we should except that actions is always of type dict, if not then erroring here is fine
             for agent, unit_actions in actions.items():
@@ -482,17 +566,25 @@ class LuxAI2022(ParallelEnv):
                         if "factory" in unit_id:
                             self.state.factories[agent][unit_id].action_queue.append(format_factory_action(action))
                         elif "unit" in unit_id:
+                            unit = self.state.units[agent][unit_id]
+                            # if unit does not have more than ACTION_QUEUE_POWER_COST power, we skip updating the action queue and print warning
+                            update_power_req = self.state.env_cfg.ROBOTS[unit.unit_type.name].ACTION_QUEUE_POWER_COST * weather_cfg["power_loss_factor"]
+                            if unit.power < update_power_req:
+                                self._log(f"{agent} Tried to update action queue for {unit} requiring ({self.state.env_cfg.ROBOTS[unit.unit_type.name].ACTION_QUEUE_POWER_COST} x {weather_cfg['power_loss_factor']}) = {update_power_req} power but only had {unit.power} power. Power cost factor is {weather_cfg['power_loss_factor']} ")
+                                continue
                             formatted_actions = []
                             if type(action) == list or (type(action) == np.ndarray and len(action.shape) == 2):
                                 trunked_actions = action[: self.env_cfg.UNIT_ACTION_QUEUE_SIZE]
                                 formatted_actions = [format_action_vec(a) for a in trunked_actions]
                             else:
-                                
-                                formatted_actions = [format_action_vec(action)]
+                                self._log(f"{agent} Tried to update action queue for {unit} but did not provide an action queue, provided {action}")
+                                failed_agents[agent] = True
+                                continue
+                            unit.power -= update_power_req
                             self.state.units[agent][unit_id].action_queue = formatted_actions
-                except ValueError as e:
+                except Exception as e:
                     # catch errors when trying to format unit or factory actions
-                    print(e)
+                    print(e.with_traceback(None))
                     failed_agents[agent] = True
         
             # 2. store actions by type
@@ -515,9 +607,14 @@ class LuxAI2022(ParallelEnv):
                     # update information for lichen growing and cache it
                     factory.cache_water_info(self.state.board, self.env_cfg)
 
-            # 3. validate all actions against current state, throw away impossible actions TODO
+            # 3. validate all actions against current state, throw away impossible actions
             actions_by_type = validate_actions(self.env_cfg, self.state, actions_by_type, verbose=self.env_cfg.verbose, weather_cfg=weather_cfg)
 
+            if self.collect_stats:
+
+                lichen_before = self.state.board.lichen.copy()
+                lichen_strains_before = self.state.board.lichen_strains.copy()
+
             self._handle_transfer_actions(actions_by_type)
             self._handle_pickup_actions(actions_by_type)
             self._handle_dig_actions(actions_by_type, weather_cfg)
@@ -529,8 +626,17 @@ class LuxAI2022(ParallelEnv):
             
             # Update lichen
             self.state.board.lichen -= 1
-            self.state.board.lichen = self.state.board.lichen.clip(0)
+            self.state.board.lichen = self.state.board.lichen.clip(0, self.env_cfg.MAX_LICHEN_PER_TILE)
             self.state.board.lichen_strains[self.state.board.lichen == 0] = -1
+            if self.collect_stats:
+                lichen_change = self.state.board.lichen - lichen_before
+                for agent in self.agents:
+                    for strain in self.state.teams[agent].factory_strains:
+                        start_of_step_lichen_tiles: np.ndarray = lichen_change[lichen_strains_before == strain]
+                        lichen_lost = start_of_step_lichen_tiles[start_of_step_lichen_tiles < 0].sum()
+                        lichen_gained = start_of_step_lichen_tiles.sum() - lichen_lost
+                        self.state.stats[agent]["lichen_grown"] += lichen_gained
+                        self.state.stats[agent]["lichen_lost"] -= lichen_lost
 
             # resources refining
             for agent in self.agents:
@@ -543,11 +649,14 @@ class LuxAI2022(ParallelEnv):
                 for factory in factories_to_destroy:
                     # destroy factories that ran out of water
                     self.destroy_factory(factory)
+                    if self.collect_stats:
+                        self.state.stats[factory.team.agent]["factories_lost"] += 1
             # power gain
             if is_day(self.env_cfg, self.state.real_env_steps):
                 for agent in self.agents:
                     for u in self.state.units[agent].values():
                         u.power = u.power + math.ceil(self.env_cfg.ROBOTS[u.unit_type.name].CHARGE * weather_cfg["power_gain_factor"])
+                        u.power = min(u.power, u.unit_cfg.BATTERY_CAPACITY)
             for agent in self.agents:
                 for f in self.state.factories[agent].values():
                     # Factories are immune to weather thanks to using nuclear reactors instead
@@ -560,16 +669,26 @@ class LuxAI2022(ParallelEnv):
         # rewards for all agents are placed in the rewards dictionary to be returned
         rewards = {}
         for agent in self.agents:
-            strain_ids = self.state.teams[agent].factory_strains
-            if failed_agents[agent]:
-                rewards[agent] = -1000
+            if agent in self.state.teams:
+                strain_ids = self.state.teams[agent].factory_strains
+                factories_left = len(self.state.factories[agent])
+                if factories_left == 0 and self.state.real_env_steps >= 0:
+                    failed_agents[agent] = True
+                    self._log(f"{agent} lost all factories")
+                if failed_agents[agent]:
+                    rewards[agent] = -1000
+                else:
+                    agent_lichen_mask = np.isin(self.state.board.lichen_strains, strain_ids)
+                    rewards[agent] = self.state.board.lichen[agent_lichen_mask].sum()
             else:
-                agent_lichen_mask = np.isin(self.state.board.lichen_strains, strain_ids)
-                rewards[agent] = self.state.board.lichen[agent_lichen_mask].sum()
+                # if this was not initialize then agent failed in step 0
+                failed_agents[agent] = True
+                rewards[agent] = -1000
 
         self.env_steps += 1
         self.state.env_steps += 1
         env_done = self.state.real_env_steps >= self.state.env_cfg.max_episode_length
+        env_done = env_done or failed_agents["player_0"] or failed_agents["player_1"] # env is done if any agent fails.
         dones = {agent: env_done or failed_agents[agent] for agent in self.agents}
 
         # generate observations
@@ -601,10 +720,13 @@ class LuxAI2022(ParallelEnv):
         factory.cargo.water = self.env_cfg.INIT_WATER_METAL_PER_FACTORY
         factory.cargo.metal = self.env_cfg.INIT_WATER_METAL_PER_FACTORY
         factory.power = self.env_cfg.INIT_POWER_PER_FACTORY
-        if not self.state.board.spawn_masks[team.agent][pos[0], pos[1]]:
-            self._log(f"{team.agent} cannot place factory at {pos[0]}, {pos[1]} as it is on the other half of map.")
+        if self.state.board.valid_spawns_mask[pos[0], pos[1]] == 0:
+            # Check if any tiles under the factory are invalid spawn tile.
+            # TODO - min distance between Factories? stone: I think it's a bad strategy to try and enclose a opponent factory anyway, 
+            # wastes a few factories and hard to maintain
+            self._log(f"{team.agent} cannot place factory at {pos[0]}, {pos[1]} as it overlaps an existing factory or is on top of a resource")
             return None
-        if self.state.board.factory_occupancy_map[factory.pos_slice].sum() >= 0:
+        if self.state.board.factory_occupancy_map[factory.pos_slice].max() >= 0:
             self._log(f"{team.agent} cannot overlap factory placement. Existing factory at {factory.pos} already.")
             return None
         
@@ -613,6 +735,13 @@ class LuxAI2022(ParallelEnv):
         self.state.factories[team.agent][factory.unit_id] = factory
         self.state.board.factory_map[self.state.board.pos_hash(factory.pos)] = factory
         self.state.board.factory_occupancy_map[factory.pos_slice] = factory.num_id
+        invalid_spawn_indices = factory.min_dist_slice
+        # TODO: perf - this can be faster
+        # self.state.board.valid_spawns_mask[factory.pos_slice] = False
+        for x,y in invalid_spawn_indices:
+            if x < 0 or y < 0 or x >= self.state.board.rubble.shape[0] or y >= self.state.board.rubble.shape[1]:
+                continue
+            self.state.board.valid_spawns_mask[x,y] = False
         self.state.board.rubble[factory.pos_slice] = 0
         self.state.board.ice[factory.pos_slice] = 0
         self.state.board.ore[factory.pos_slice] = 0
@@ -624,10 +753,12 @@ class LuxAI2022(ParallelEnv):
         """
         # NOTE this doesn't remove unit reference from board map
         """
-        self.state.board.rubble[unit.pos.y, unit.pos.x] = min(
-            self.state.board.rubble[unit.pos.y, unit.pos.x] + unit.unit_cfg.RUBBLE_AFTER_DESTRUCTION,
+        self.state.board.rubble[unit.pos.x, unit.pos.y] = min(
+            self.state.board.rubble[unit.pos.x, unit.pos.y] + unit.unit_cfg.RUBBLE_AFTER_DESTRUCTION,
             self.env_cfg.MAX_RUBBLE,
         )
+        self.state.board.lichen[unit.pos.x, unit.pos.y] = 0
+        self.state.board.lichen_strains[unit.pos.x, unit.pos.y] = -1
         del self.state.units[unit.team.agent][unit.unit_id]
 
     def destroy_factory(self, factory: Factory):
@@ -637,9 +768,6 @@ class LuxAI2022(ParallelEnv):
         self.state.board.factory_occupancy_map[factory.pos_slice] = -1
         del self.state.factories[factory.team.agent][factory.unit_id]
         del self.state.board.factory_map[self.state.board.pos_hash(factory.pos)]
-        # TODO - shouldn't rubble under factory always be 0?
-        # TODO remove units on each factory tile
-
 
 def raw_env() -> LuxAI2022:
     """
